Exception: Kumi::Core::Errors::ConfigurationError

Inherits:
Error
  • Object
show all
Defined in:
lib/kumi/core/errors.rb

Overview

Kumi was used or configured incorrectly by the host application: no schema defined, an unknown compilation mode, a schema not precompiled in :aot mode. The schema itself may be fine — the call site or configuration is wrong, so the message tells the operator what to do.